from __future__ import print_function

import click

import os
import shutil
import sys
import tempfile

from obsctl.cli import pass_context
from obsctl.lib import obsc
from obsctl.lib import spec
from obsctl.lib import tar


@click.command('tarimport', help='Push tarball of sources to an OBS instance.')
@click.option('--source-file', '-f', help='Path to source tarball file', required=True)
@click.option('--continuous-merge', '-m', help='Enable continuous merge mode',
              is_flag=True)
@click.option('--repo-type', '-T', help='Source repository type',
              type=click.Choice(['git', 'hg', 'svn']), required=True)
@click.option('--repo-owner', '-o', help='Source repository owner',
              required=True)
@click.option('--repo-name', '-n', help='Source repository name')
@click.option('--repo-reltag', '-t', help='Source repository release tag',
              required=True)
@click.option('--repo-revision', '-R',
              help='Source repository revision/commit', required=True)
@click.option('--repo-branch', '-B', help='Source repository branch',
              required=True)
@click.option('--revision-form', '-r', help='Form of version for spec',
              type=click.Choice(['prerel', 'postrel']), default='postrel')
@click.option('--obs-project', '-s', help='Open Build Service project space',
              required=True)
@click.option('--obs-package', '-p', help='Open Build Service project package',
              required=True)
@click.option('--create-obs-pkg-if-none', '-c',
              help='Create package space in OBS if it doesn\'t exist',
              is_flag=True)
@click.option('--insecure', '-i',
              help='Ignore issues with HTTPS (self signed certs, etc.)',
              is_flag=True)
@pass_context
def cli(ctx, source_file, continuous_merge, repo_type, repo_owner, repo_name,
        repo_reltag, repo_revision, repo_branch, revision_form, obs_project,
        obs_package, create_obs_pkg_if_none, insecure):
    # TODO: Clean this up
    args = {
            'src_tarball': source_file,
            'ci_mode': continuous_merge,
            'src_repo_type': repo_type,
            'src_repo_owner': repo_owner,
            'src_repo_name': repo_name,
            'src_repo_tag': repo_reltag,
            'src_repo_rev': repo_revision,
            'src_repo_branch': repo_branch,
            'spec_rev_form': revision_form,
            'obs_server_addr': ctx.obs_server,
            'obs_auth_user': ctx.obs_user,
            'obs_auth_pass': ctx.obs_pass,
            'obs_project': obs_project,
            'obs_package': obs_package,
            'create_obs_pkg_if_none': create_obs_pkg_if_none,
            'insecure': insecure
        }

    # Configure the Open Build Service Commander
    oscconf = obsc.configure(args['obs_server_addr'],
                             args['obs_auth_user'],
                             args['obs_auth_pass'],
                             args['insecure'])

    obspkg = obsc.checkout_obs_package(oscconf,
                                       args['obs_server_addr'],
                                       args['obs_project'],
                                       args['obs_package'],
                                       args['obs_auth_user'],
                                       args['src_repo_owner'],
                                       args['src_repo_name'],
                                       args['create_obs_pkg_if_none'])

    if obspkg is False:
        print("No package space exists, exiting!")
        sys.exit(2)

    # Generate a snapshot version
    spec_snap_version = spec.generate_snapshot_version(args['src_repo_tag'],
                                                       args['src_repo_type'],
                                                       args['src_repo_rev'],
                                                       args['src_repo_branch'],
                                                       args['spec_rev_form'])

    if args['ci_mode'] is True:
        release_tarball_ver = spec_snap_version
    else:
        release_tarball_ver = args['src_repo_tag']

    rebuilt_tarball_name = tar.rebuild_tarball(args['src_tarball'],
                                               args['src_repo_name'],
                                               release_tarball_ver)

    print("{0} has been rebuilt.".format(rebuilt_tarball_name))

    # Auto-set the version
    if args['ci_mode'] is True:
            obs_version_service_chunk = """
      <service name="set_version">
        <param name="version">{0}</param>
        <param name="file">_service:extract_file:{1}.spec</param>
      </service>""".format(release_tarball_ver, args['obs_package'])
    else:
            obs_version_service_chunk = ""

    # Service file for retrieving the spec from the tarball
    obs_package_service = """
    <services>
      <service name="extract_file">
        <param name="archive">{0}*.tar.gz</param>
        <param name="files">*{0}*/dist/{1}.spec</param>
      </service>{2}
    </services>
    """.format(args['src_repo_name'], args['obs_package'],
               obs_version_service_chunk)

    # Write service file out

    obs_package_service_file = tempfile.NamedTemporaryFile(suffix="_service")

    obs_package_service_file.write(obs_package_service)
    obs_package_service_file.flush()

    # Send changes to OBS
    obsc.update_files_for_obs_package(obspkg, release_tarball_ver,
                                      [rebuilt_tarball_name],
                                      obs_package_service_file.name)

    # Purge working things
    shutil.rmtree(os.path.dirname(rebuilt_tarball_name))
    obs_package_service_file.close()
